#ifndef MAPPER_H
#define MAPPER_H

/**
 * @file mapper.h
 * @brief Defines mapper task's data structures and functionality.
 */

#include <mapreduce/input_reader.h>
#include <mapreduce/intermediate_writer.h>

/**
 * @struct mapper_data
 * @brief Mapper's data structure.
 */
struct mapper_data {
	struct mapreduce_input_reader in_r;
	struct mapreduce_intermediate_writer im_w;
	struct partitions parts;
};

/**
 * @struct mapper_env
 * @brief Mapper's working environment for particular
 *        map-reduce execution.
 */
struct mapper_env {
	int in_fd;
	int im_fd;
	int part_fd;
	size_t partitions_num;
	bool (*map)(struct mapreduce_input_reader* in_r,
		    struct mapreduce_intermediate_writer* im_w);
};

// Main function of Mapper process
void mapper_task(const struct mapper_env* environment,struct mapreduce_error* err);

#endif // MAPPER_H
