#ifndef PARTITIONS_H
#define PARTITIONS_H

/**
 * @file partitions.h
 * @brief Defines partitions structure and declares its
 *        construction/destruction functions.
 */

#include <stdlib.h>

#include <fcntl.h>

/**
 * @struct partitions
 * @brief Associates partition numbers with partition details.
 */
struct partitions {
	struct partition* parts;
	size_t parts_num;
};

/**
 * @struct partition
 * @brief Describes offset and size of partition inside
 *        intermediate file.
 */
struct partition {
	off_t offset;
	size_t size;
};

/**
 * @brief Constructor.
 * @param parts_num Number of partitions to describe
 *                  with returned object.
 * @return Fully initialized object.
 */
struct partitions partitions_init(size_t parts_num);

/**
 * @brief Destructor.
 * @param destroyed Object to deinitialize.
 */
void partitions_destroy(struct partitions* destroyed);

#endif // PARTITIONS_H
