#ifndef REDUCER_H
#define REDUCER_H

/**
 * @file reducer.h
 * @brief Defines reducer task's data structures and functionality.
 */

#include <mapreduce/intermediate_reader.h>
#include <mapreduce/output_writer.h>
#include <mapreduce/task.h>

/**
 * @struct reducer_data
 * @brief Reducer's data structure.
 */
struct reducer_data {
	struct mapreduce_intermediate_reader im_r;
	struct mapreduce_output_writer out_w;
	struct task reduce_task;
};

/**
 * @struct reducer_env
 * @brief Reducer's working environment for particular
 *        map-reduce execution.
 */
struct reducer_env {
	int out_fd;
	int tasks_fd;
	int tasks_sem;
	const char** im_filenames;
	size_t im_num;
	bool (*reduce)(struct mapreduce_intermediate_reader* im_r,
		       struct mapreduce_output_writer* out_w);
};

// Main function of Reducer process
void reducer_task(const struct reducer_env* environment,struct mapreduce_error* err);

#endif // REDUCER_H
