/**
 * @file LibraryTest.hpp
 * @brief File contains declarations of LibraryTest class.
 */

#ifndef LIBRARYTEST_HPP
#define LIBRARYTEST_HPP

#include <mapreduce/tests/AbstractTest.hpp>
#include <mapreduce/input_reader.h>
#include <mapreduce/intermediate_writer.h>
#include <mapreduce/intermediate_reader.h>
#include <mapreduce/output_writer.h>
#include <mapreduce/partitions.h>

#include <fstream>
#include <iostream>
#include <cstring>


/**
 * Map reduce test project namespace.
 */
namespace MapReduceTest
{
	/**
	 * Class represents MapReduce library functions unit test suite class.
	 * It is used to run tests of library file I/O functions.
	 */
	class LibraryTest : public AbstractTest
	{
	private:
		struct mapreduce_input_reader inReader;
		struct mapreduce_intermediate_writer imWriter;
		struct mapreduce_intermediate_reader imReader;
		struct mapreduce_output_writer outWriter;

	public:
		/**
 		 * Default constructor.
		 */
		LibraryTest() {}
		
		/**
 		 * Default destructor.
		 */
		~LibraryTest() {}

		void runInputReaderTest();
		void runIntermediateWriterTest();
		void runIntermediateReaderTest();
		void runOutputWriterTest();

		void runTest();
	};
}
#endif // LIBRARYTEST_HPP

