/**
 * @file logger.h
 * @brief Defines functions for map_reduce lib logger structure.
 */

#define _XOPEN_SOURCE 500 // for fdopen() in stdio.h
#include <stdio.h>
#include <stdlib.h>
#include <iso646.h>
#include <time.h>

#include <unistd.h>
#include <fcntl.h>
#include <sys/sem.h>
#include <sys/ipc.h>

#include <mapreduce/logger.h>

/**
 * @struct logger_data
 * @brief Logger's data structure.
 */
static struct logger_data{
	FILE* fd;
	int mutex;
} logger;

static int mutex_init();
static void mutex_destroy(const int mutex);
static void P(const int sem);
static void V(const int sem);

void
logger_init()
{
        logger.fd = fopen("log.log", "w");
	logger.mutex = mutex_init();
	V(logger.mutex);
	logger_event("Logger initialized\n");
}

void
logger_destroy()
{
	logger_event("Logger destroyed\n");
	mutex_destroy(logger.mutex);
	fclose(logger.fd);
}

void
logger_event(const char* const log)
{	
	#ifdef LOGGER
	P(logger.mutex);
	time_t rawtime;
  	struct tm* timeinfo;
  	char time_buffer[80];

	time (&rawtime);
  	timeinfo = localtime (&rawtime);
  	strftime(time_buffer,80,"%c ",timeinfo);

	fprintf(logger.fd, "pid: %i\tdate: %s\tmsg: %s\t\n", getpid(), time_buffer, log);
	fflush(logger.fd);
	V(logger.mutex);
	#endif
}

static int
mutex_init()
{
	const int mutex = semget(IPC_PRIVATE, 1, 0666|IPC_CREAT);
	if (mutex == -1) {
		printf("Semaphore init error occured\n");
		exit(1);
	}
	return mutex;
}
	
static void
mutex_destroy(const int mutex)
{
	if (semctl(mutex, 0, IPC_RMID) == -1) {
		printf("Semaphore destroy error occurred\n");
		exit(1);
	}
}

static void
P(const int sem)
{
	struct sembuf op = {
		.sem_num = 0,
		.sem_op = -1,
		.sem_flg = SEM_UNDO
	};
	semop(sem, &op, 1);
}
	
static void
V(const int sem)
{
	struct sembuf op = {
		.sem_num = 0,
		.sem_op = +1,
		.sem_flg = SEM_UNDO
	};
	semop(sem, &op, 1);
}
