/**
 * @file mapper.c
 * @brief Implements mapper task's functionality.
 */

#include <stdlib.h>
#include <iso646.h>

#include <unistd.h>

#include <mapreduce/mapper.h>
#include <mapreduce/logger.h>

/**
 * @brief Helper functions declarations.
 */

//Initialize Mapper data structure by using mapper_env strucutre from Master process
static bool prepare_mapper(const struct mapper_env* const environment, struct mapper_data* const data);

//Saves created partitions in mapping process to Mapper Partition file.
static bool save_partitions(const struct mapper_env* const environment, struct mapper_data* const data);

static bool
prepare_mapper(const struct mapper_env* const env, struct mapper_data* const data)
{
	data->parts = partitions_init(env->partitions_num);
	data->in_r.fd = env->in_fd;
	data->im_w.fd = env->im_fd;
	data->im_w.parts = data->parts;	
	return true;
}

static bool
save_partitions(const struct mapper_env* const env, struct mapper_data* const data)
{
	for (size_t i = 0; i < env->partitions_num; ++i) {
		const ssize_t size_write = write( \
			env->part_fd, &data->im_w.parts.parts[i], \
			sizeof data->im_w.parts.parts[0] \
		);
		if(size_write not_eq sizeof data->im_w.parts.parts[0])
			return false;
	}
	return true;
}

void
mapper_task(const struct mapper_env* const env, struct mapreduce_error* const err) {
	logger_event("Mapper task executed\n");	
	struct mapper_data data;
	if (not prepare_mapper(env,&data)) {
		logger_event("Mapper prepare error occurred!\n");
		err->code = MAPREDUCE_ERROR_INTERNAL;
		return;
	}
	if (not env->map(&data.in_r,&data.im_w)) {
		logger_event("Mapper map() error occurred!\n");
		err->code = MAPREDUCE_ERROR_MAP_FAILURE;
		return;
	}
	logger_event("Mapper mapping task task ended\n");
	if (not save_partitions(env,&data)) {
		logger_event("Mapper save partitions error occurred\n");
		err->code = MAPREDUCE_ERROR_INTERNAL;
		return;
	}
	logger_event("Mapper mapper_task success\n");
}

