/**
 * @file CrashTest.cpp
 * @brief File contains definition of CrashSearch test class.
 */

#include <mapreduce/tests/CrashTest.hpp>


using namespace std;

namespace MapReduceCrashTest
{

  bool
  null_map(struct mapreduce_input_reader* const, struct mapreduce_intermediate_writer* const)
  {
    bool* const deadly_pointer = NULL;
    return *deadly_pointer;
  }

  bool
  null_reduce(struct mapreduce_intermediate_reader* const, struct mapreduce_output_writer* const)
  {
    bool* const deadly_pointer = NULL;
    return *deadly_pointer;
  }

  bool
  map(struct mapreduce_input_reader* const, struct mapreduce_intermediate_writer* const)
  {
    return true;
  }

  bool
  reduce(struct mapreduce_intermediate_reader* const, struct mapreduce_output_writer* const)
  {
    return true;
  }

  bool
  fail_map(struct mapreduce_input_reader* const, struct mapreduce_intermediate_writer* const)
  {
    return false;
  }

  bool
  fail_reduce(struct mapreduce_intermediate_reader* const, struct mapreduce_output_writer* const)
  {
    return false;
  }

  bool
  empty_map(struct mapreduce_input_reader* const, struct mapreduce_intermediate_writer* const im_w)
  {
    size_t write = 0;
    const bool ok = mapreduce_write_part(im_w, "", &write, 0);
    return ok;
  }

  bool
  empty_reduce(struct mapreduce_intermediate_reader* const im_r, struct mapreduce_output_writer* const)
  {
    size_t read = 0;
    char buffer[1];
    const bool ok = mapreduce_read_intermediate(im_r, buffer, &read);
    return ok;
  }

  CrashTest::CrashTest()
    : inputs("crash.in"), outputs("crash.out")
  {
	  spec.inputs = &inputs;
	  spec.outputs = &outputs;
	  spec.inputs_num = 1;
	  spec.outputs_num = 1;
  }

  void
  CrashTest::runTest()
  {
	  FILE* const input = fopen(inputs, "w");
	  fclose(input);
	  null_map_test();
	  null_reduce_test();
	  empty_input_output_test();
	  fail_map_test();
	  fail_reduce_test();
  }

  void
  CrashTest::null_map_test()
  {
	  spec.map = &null_map;
	  spec.reduce = &reduce;
	  if (mapreduce(&spec, &error) == false)
	  {
	    if (error.code == MAPREDUCE_ERROR_MAP_TERMINATION)
	      cout << "CrashTest with map() termination ok" << endl;
	    else
	      cout << "CrashTest with map() termination detected wrong type of error: "
		   << error.code << endl;
	  }
	  else
	  {
	    cout << "CrashTest with map() termination succeded unexpectedly" << endl;
	  }
  }

  void
  CrashTest::null_reduce_test()
  {
	  spec.map = &map;
	  spec.reduce = &null_reduce;
	  if (mapreduce(&spec, &error) == false)
	  {
	    if (error.code == MAPREDUCE_ERROR_REDUCE_TERMINATION)
	      cout << "CrashTest with reduce() termination ok" << endl;
	    else
	      cout << "CrashTest with reduce() termination detected wrong type of error: "
		   << error.code << endl;
	  }
	  else
	  {
	    cout << "CrashTest with reduce() termination unexpectedly succeded" << endl;
	  }
  }

  void
  CrashTest::empty_input_output_test()
  {
	  spec.map = &empty_map;
	  spec.reduce = &empty_reduce;
	  if (mapreduce(&spec, &error))
	  {
	    cout << "CrashTest with empty input/output ok" << endl;
	  }
	  else
	  {
	    cout << "CrashTest with empty input/output failed with code: " << error.code << endl;
	  }
  }

  void
  CrashTest::fail_map_test()
  {
	  spec.map = &fail_map;
	  spec.reduce = &reduce;
	  if (mapreduce(&spec, &error) == false)
	  {
	    if (error.code == MAPREDUCE_ERROR_MAP_FAILURE)
	      cout << "CrashTest with map() failure ok" << endl;
	    else
	      cout << "CrashTest with map() failure detected wrong type of error: "
		   << error.code << endl;
	  }
	  else
	  {
	    cout << "CrashTest with map() failure succeded unexpectedly" << endl;
	  }
  }

  void
  CrashTest::fail_reduce_test()
  {
	  spec.map = &map;
	  spec.reduce = &fail_reduce;
	  if (mapreduce(&spec, &error) == false)
	  {
	    if (error.code == MAPREDUCE_ERROR_REDUCE_FAILURE)
	      cout << "CrashTest with reduce() failure ok" << endl;
	    else
	      cout << "CrashTest with reduce() failure detected wrong type of error: "
		   << error.code << endl;
	  }
	  else
	  {
	    cout << "CrashTest with reduce() failure succeded unexpectedly" << endl;
	  }
  }
}
