/**
 * @file LibraryTest.cpp
 * @brief File contains definition of LibraryTest test class.
 */

#include <mapreduce/tests/LibraryTest.hpp>

using namespace std;

namespace MapReduceTest
{
	
	/**
	 * Method is used to run test.
	 */
	void
	LibraryTest::runTest()
	{
		runInputReaderTest();
		runIntermediateWriterTest();
		runIntermediateReaderTest();
		runOutputWriterTest();
	}

	/**
         * Method is used to run MapReduce library input_reader read function.
 	 */
	void
	LibraryTest::runInputReaderTest()
	{
		FILE* file;
		int fd;
		char buffer[40];
		char buffer2[40];
		size_t buffer_size = (sizeof buffer)/(sizeof buffer[0]);

		file = fopen("test.in", "wb");
		fwrite("Jestem fajny\n",1,sizeof("Jestem fajny\n"), file);
		fclose(file);

		file = fopen("test.in","rb");
		fd = open("test.in", O_RDONLY);

		inReader.fd = fd;

		fread(&buffer,1,buffer_size,file);

		const int read_success = mapreduce_read_input(&inReader,&buffer2,&buffer_size); 

		if(not read_success or memcmp(&buffer,&buffer2,buffer_size)) 
		{
			cout << "InputReader test failed! " << endl;
			return;
		}
		
		cout << "InputReader test ok! " << endl;

		close(fd);
		fclose(file);
	}

	/**
	 * Method is used to test MapReduce library intermediate_writer write function.
	 */
	void
	LibraryTest::runIntermediateWriterTest()
	{
		FILE* file;
		int fd;
		char buffer[40];
		char buffer2[40];
		size_t buffer_size = (sizeof buffer)/(sizeof buffer[0]);

		file = fopen("test.in", "wb");
		fwrite("Jestem fajny\n",1,sizeof("Jestem fajny\n"), file);
		fclose(file);
		
		file = fopen("test.in","rb");
		fread(&buffer,1,buffer_size,file);
		fclose(file);
				
		fd = open("test.in", O_WRONLY);
		imWriter.fd = fd;

		imWriter.parts.parts_num = 1;
		imWriter.parts.parts = new struct partition[2];

		size_t size = sizeof("Jestem fajny\n");
		int write_success =  mapreduce_write_part(&imWriter,"Jestem fajny\n",&size,0);		
		close(fd);

		file = fopen("test.in","rb");
		fread(&buffer2,1,buffer_size,file);
			
		if(not write_success or memcmp(&buffer,&buffer2,size))
		{
			cout << "IntermediateWriter test failed! " << endl;
			return;
		}

		cout << "IntermediateWriter test ok! " << endl;
		fclose(file);
	}

	/**
	 * Method is used to test MapReduce library intermediate_reader read function.
	 */
	void
	LibraryTest::runIntermediateReaderTest()
	{
		FILE* file;
		int fd;
		char buffer[40];
		char buffer2[40];
		size_t buffer_size = (sizeof buffer)/(sizeof buffer[0]);

		file = fopen("test.in", "wb");
		fwrite("Jestem fajny\n",1,sizeof("Jestem fajny\n"), file);
		fclose(file);

		file = fopen("test.in","rb");
		fd = open("test.in", O_RDONLY);

		imReader.fd = fd;

		fread(&buffer,1,buffer_size,file);

		const int read_success = mapreduce_read_intermediate(&imReader,&buffer2,&buffer_size); 

		if(not read_success or memcmp(&buffer,&buffer2,buffer_size)) 
		{
			cout << "IntermediateReader test failed! " << endl;
			return;
		}
		
		cout << "IntermediateReader test ok! " << endl;

		close(fd);
		fclose(file);
	}

	/**
         * Method is used to run MapReduce library output_writer write function.
 	 */
	void
	LibraryTest::runOutputWriterTest()
	{
		FILE* file;
		int fd;
		char buffer[40];
		char buffer2[40];
		size_t buffer_size = (sizeof buffer)/(sizeof buffer[0]);

		file = fopen("test.in", "wb");
		fwrite("Jestem fajny\n",1,sizeof("Jestem fajny\n"), file);
		fclose(file);
		
		file = fopen("test.in","rb");
		fread(&buffer,1,buffer_size,file);
		fclose(file);
				
		fd = open("test.in", O_WRONLY);
		outWriter.fd = fd;

		size_t size = sizeof("Jestem fajny\n");
		int write_success = mapreduce_write_out(&outWriter,"Jestem fajny\n",&size);		
		close(fd);

		file = fopen("test.in","rb");
		fread(&buffer2,1,buffer_size,file);
			

		if(not write_success or memcmp(&buffer,&buffer2,size))
		{
			cout << "OutputWriter test failed! " << endl;
			return;
		}
		
		cout << "OutputWriter test ok! " << endl;
		fclose(file);
	}
}

